<?php

namespace Drupal\language_switcher_menu;

use Drupal\Core\Menu\DefaultMenuLinkTreeManipulators;
use Drupal\Core\Menu\MenuLinkInterface;
use Drupal\Core\Access\AccessResult;
use Drupal\language_switcher_menu\Plugin\Menu\LanguageSwitcherLink;

/**
 * Extends access check tree manipulator provided by Drupal Core.
 *
 * @todo Remove once https://www.drupal.org/project/drupal/issues/3008889 has
 *   been fixed.
 */
class LanguageLinkAccessMenuTreeManipulator extends DefaultMenuLinkTreeManipulators {

  /**
   * {@inheritdoc}
   */
  protected function menuLinkCheckAccess(MenuLinkInterface $instance) {
    $access_result = parent::menuLinkCheckAccess($instance);
    if (!$access_result->isNeutral()) {
      return $access_result;
    }

    if ($instance->getUrlObject()->getRouteName() !== '<current>') {
      return $access_result;
    }
    // @note Third party modules may alter language switch links and they would
    // still be an instance of our class, so we check for it relatively late.
    if (!$instance instanceof LanguageSwitcherLink) {
      return $access_result;
    }
    $has_access = $this->account->hasPermission('view language_switcher_menu links');
    return AccessResult::allowedIf($has_access)->cachePerPermissions();
  }

}
